@@ -17,6 +17,7 @@
 	(C) Heikki Holstila 2010
 */
 
+#include <QtCore>
 #include <iostream>
 #include "aptinterface.h"
 #include "mainwindow.h"
@@ -29,6 +30,7 @@ AptInterface::AptInterface(MainWindow* w_)
 	iRunProc = new QProcess();
 	connect(iRunProc,SIGNAL(error(QProcess::ProcessError)),this,SLOT(procError(QProcess::ProcessError)));
 	connect(iRunProc,SIGNAL(finished(int,QProcess::ExitStatus)),this,SLOT(procFinished(int,QProcess::ExitStatus)));
+	iRunProc->setProcessChannelMode(QProcess::MergedChannels);
 
 	iMode = ModeNone;
 	iReady = false;
@@ -59,6 +61,11 @@ void AptInterface::updatePackageList()
 	toRemove.clear();
 	pkgNamesList.clear();
 
+	QDir logdir(KlogFileDir);
+	logdir.mkpath(KlogFileDir);
+	QFile logfile(KlogFileName);
+	logfile.remove();
+
 	QHashIterator<QString, Package*> i( iPackages );
 	while (i.hasNext()) {
 		i.next();
@@ -86,6 +93,9 @@ bool AptInterface::isInterfaceReady()
 
 void AptInterface::procError(QProcess::ProcessError error)
 {
+	QByteArray output = iRunProc->readAllStandardOutput();
+	logToFile(output);
+
 	iError.setNum(error);
 	iError.prepend(tr("QProcess error "));
 	iMainWindow->operationFinished(iMode, false, iError);
@@ -97,18 +107,36 @@ void AptInterface::procFinished(int exitCode, QProcess::ExitStatus exitStatus)
 	aptMode lastMode = iMode;
 	iMode = ModeNone;
 	if( exitCode != 0 ) {
-		iError.setNum(exitCode);
-		iError.prepend(tr("Exit code "));
-		QString s;
-		s.setNum(exitStatus);
-		iError.append(", status " + s);
-		iMainWindow->operationFinished(lastMode, false, iError);
+		QByteArray output = iRunProc->readAllStandardOutput();
+		logToFile(output);
+
+		if( output.contains("Could not get lock") ) {
+			iMainWindow->operationFinished(lastMode, false, "<b>Could not get lock</b><br>" \
+										   "The package management system is locked by another process");
+		} else if( output.contains("E: Unable to fetch some archives") ) {
+			iMainWindow->operationFinished(lastMode, false, "<b>Failed to fetch packages</b><br>" \
+										   "Your network connection might be down or your catalogs could be out of date");
+		} else {
+			iError.setNum(exitCode);
+			iError.prepend(tr("Exit code "));
+			QString s;
+			s.setNum(exitStatus);
+			iError.append(", status " + s);
+			iMainWindow->operationFinished(lastMode, false, iError);
+		}
 	} else {
 
 		if( lastMode == ModeAptGetUpdate )
 		{
+			QByteArray output = iRunProc->readAllStandardOutput();
+			logToFile(output);
 			iRunProc->close();
-			iMainWindow->operationFinished(lastMode, true, "");
+			if( output.contains("Could not resolve host") ) {
+				iMainWindow->operationFinished(lastMode, false, "<b>Could not resolve host</b><br>" \
+											   "Some (or all) of the catalogs might be incomplete. Check your network connection");
+			} else {
+				iMainWindow->operationFinished(lastMode, true, "");
+			}
 		}
 
 		if( lastMode == ModeReadInfo )
@@ -141,6 +169,9 @@ void AptInterface::procFinished(int exitCode, QProcess::ExitStatus exitStatus)
 
 		if( lastMode == ModeAptGetInstall || lastMode == ModeAptGetRemove )
 		{
+			QByteArray output = iRunProc->readAllStandardOutput();
+			logToFile(output);
+
 			iRunProc->close();
 			iMainWindow->operationFinished(lastMode, true, "");
 		}
@@ -165,7 +196,7 @@ bool AptInterface::parseAptCacheOutput()
 			QString pkgname = lines.at(i).mid(9).trimmed();
 			if( pkgname==0 || pkgname == "" ) {
 				iError = tr("Empty package name");
-				std::cout << "Encountered empty package name!" << std::endl;
+				//std::cout << "Encountered empty package name!" << std::endl;
 				return false;
 			}
 			pkg = new Package(pkgname);
@@ -192,7 +223,7 @@ bool AptInterface::parseAptCacheOutput()
 			pkg->setDescShort(ds);
 		}
 	}
-	std::cout << "apt-cache: read " << iPackages.count() << " available packages" << std::endl;
+	//std::cout << "apt-cache: read " << iPackages.count() << " available packages" << std::endl;
 	return true;
 }
 
@@ -217,7 +248,7 @@ bool AptInterface::readDpkgDatabase()
 			QString pkgname = line.mid(9).trimmed();
 			if( pkgname==0 || pkgname == "" ) {
 				iError = tr("Empty package name");
-				std::cout << "Encountered empty package name!" << std::endl;
+				//std::cout << "Encountered empty package name!" << std::endl;
 				return false;
 			}
 			pkg = iPackages.value(pkgname,0);
@@ -255,14 +286,16 @@ bool AptInterface::readDpkgDatabase()
 
 	db.close();
 
-	std::cout << "Read dpkg database info for " << totpkg << " packages (" << newpkg << " new)" << std::endl;
+	//std::cout << "Read dpkg database info for " << totpkg << " packages (" << newpkg << " new)" << std::endl;
 
 	return true;
 }
 
 bool AptInterface::parseSimulated()
 {
-	QList<QByteArray> lines = iRunProc->readAllStandardOutput().split('\n');
+	QByteArray output = iRunProc->readAllStandardOutput();
+	logToFile(output);
+	QList<QByteArray> lines = output.split('\n');
 
 	toInstall.clear();
 	toRemove.clear();
@@ -280,6 +313,7 @@ bool AptInterface::parseSimulated()
 		}
 	}
 
+	/*
 	std::cout << "Simulated results:" << std::endl;
 	std::cout << "  Install: ";
 	for(int x=0; x<toInstall.count(); x++)
@@ -288,6 +322,7 @@ bool AptInterface::parseSimulated()
 	for(int x=0; x<toRemove.count(); x++)
 		std::cout << toRemove.at(x).toStdString() << " ";
 	std::cout << std::endl;
+	*/
 
 	if( toInstall.count()==0 && toRemove.count()==0 )
 		return false;
@@ -299,12 +334,14 @@ void AptInterface::simulateInstall(QStringList names_)
 {
 	pkgNamesList = names_;
 	iMainWindow->busyDialog(true, "Operation in progress", "Reading dependencies");
+	/*
 	std::cout << "SIMULATE apt-get install: ";
 	for( int i=0; i<names_.count(); i++)
 	{
 		std::cout << names_.at(i).toStdString() + " ";
 	}
 	std::cout << std::endl;
+	*/
 
 	iMode = ModeAptGetSimulateInstall;
 
@@ -320,12 +357,14 @@ void AptInterface::simulateRemove(QStringList names_)
 {
 	pkgNamesList = names_;
 	iMainWindow->busyDialog(true, "Operation in progress", "Reading dependencies");
+	/*
 	std::cout << "SIMULATE apt-get remove: ";
 	for( int i=0; i<names_.count(); i++)
 	{
 		std::cout << names_.at(i).toStdString() + " ";
 	}
 	std::cout << std::endl;
+	*/
 
 	iMode = ModeAptGetSimulateRemove;
 
@@ -340,18 +379,20 @@ void AptInterface::simulateRemove(QStringList names_)
 void AptInterface::installPackages(QStringList names_)
 {
 	iMainWindow->busyDialog(true, "Operation in progress", "Downloading and installing packages");
+	/*
 	std::cout << "apt-get install ";
 	for( int i=0; i<names_.count(); i++)
 	{
 		std::cout << names_.at(i).toStdString() + " ";
 	}
 	std::cout << std::endl;
+	*/
 
 	iMode = ModeAptGetInstall;
 
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
-	runParameters << "-qy" << "install";
+	runParameters << "-qy" << "--force-yes" << "install";
 	runParameters << names_;
 
 	iRunProc->start(runBinary,runParameters);
@@ -360,18 +401,20 @@ void AptInterface::installPackages(QStringList names_)
 void AptInterface::removePackages(QStringList names_)
 {
 	iMainWindow->busyDialog(true, "Operation in progress", "Removing packages");
+	/*
 	std::cout << "apt-get remove ";
 	for( int i=0; i<names_.count(); i++)
 	{
 		std::cout << names_.at(i).toStdString() + " ";
 	}
 	std::cout << std::endl;
+	*/
 
 	iMode = ModeAptGetRemove;
 
 	QString runBinary = "/usr/bin/apt-get";
 	QStringList runParameters;
-	runParameters << "-qy" << "remove";
+	runParameters << "-qy" << "--force-yes" << "remove";
 	runParameters << names_;
 
 	iRunProc->start(runBinary,runParameters);
@@ -380,7 +423,7 @@ void AptInterface::removePackages(QStringList names_)
 void AptInterface::aptGetClean()
 {
 	iMainWindow->busyDialog(true, "Operation in progress", "Cleaning up");
-	std::cout << "apt-get clean" << std::endl;
+	//std::cout << "apt-get clean" << std::endl;
 
 	iMode = ModeAptGetClean;
 
@@ -390,3 +433,87 @@ void AptInterface::aptGetClean()
 
 	iRunProc->start(runBinary,runParameters);
 }
+
+void AptInterface::logToFile( QByteArray data )
+{
+	QFile f(KlogFileName);
+
+	if( f.open( QIODevice::Append | QIODevice::WriteOnly | QIODevice::Text ) )
+	{
+		QTextStream out(&f);
+		out << data;
+		f.close();
+	}
+}
+
+QStringList AptInterface::getRepositories()
+{
+	QStringList repos;
+
+	QFile own("/root/.fapman/repos.list");
+	if( own.open(QIODevice::ReadOnly | QIODevice::Text ) )
+	{
+		while(!own.atEnd())
+		{
+			QString line = own.readLine();
+			repos.append(line.trimmed());
+		}
+		own.close();
+
+		return repos;
+	}
+
+	QFile ham("/etc/apt/sources.list.d/hildon-application-manager.list");
+	if( ham.open(QIODevice::ReadOnly | QIODevice::Text ) )
+	{
+		while(!ham.atEnd())
+		{
+			QString line = ham.readLine();
+			repos.append(line.trimmed());
+		}
+		ham.close();
+	}
+
+	QStringList additional;
+	additional
+			<< "deb http://repository.maemo.org/extras/ fremantle-1.2 free non-free"
+			<< "deb http://repository.maemo.org/extras-testing/ fremantle free non-free"
+			<< "deb http://repository.maemo.org/extras-devel/ fremantle free non-free";
+
+	for( int i=0; i<additional.count(); i++ )
+	{
+		if( !repos.contains( additional.at(i) ) )
+		{
+			repos.append( "#" + additional.at(i) );
+		}
+	}
+
+	return repos;
+}
+
+bool AptInterface::writeRepositories(QStringList repos)
+{
+	QFile own("/root/.fapman/repos.list");
+	if( own.open(QIODevice::WriteOnly | QIODevice::Text ) )
+	{
+		QTextStream out(&own);
+		for( int i=0; i<repos.count(); i++ )
+			out << repos.at(i) << "\n";
+		own.close();
+	}
+
+	QFile ham("/etc/apt/sources.list.d/hildon-application-manager.list");
+	if( ham.open(QIODevice::WriteOnly | QIODevice::Text ) )
+	{
+		QTextStream out(&ham);
+		for( int i=0; i<repos.count(); i++ )
+			out << repos.at(i) << "\n";
+
+		ham.close();
+
+		return true;
+	}
+
+	return false;
+}
+
