@@ -26,6 +26,11 @@ class MainWindow;
 class PackageView;
 class Package;
 
+
+const QString KlogFileDir = "/root/.fapman";
+const QString KlogFileName = "/root/.fapman/lastlog.txt";
+
+
 class AptInterface : public QObject
 {
 	Q_OBJECT
@@ -51,6 +56,12 @@ public:
 	void getSimulatedPackageLists(QStringList& inst_, QStringList& remv_) { inst_=toInstall; remv_=toRemove; }
 	void getOriginalPackageList(QStringList& names_) { names_=pkgNamesList; }
 
+	void logToFile( QByteArray data );
+
+	QStringList getRepositories(void);
+	bool writeRepositories(QStringList repos);
+
+
 	enum aptMode { ModeNone, ModeReadInfo, ModeAptGetUpdate, ModeAptGetInstall, ModeAptGetRemove,
 				   ModeAptGetSimulateInstall, ModeAptGetSimulateRemove, ModeAptGetClean };
 
