@@ -24,15 +24,12 @@
 #include <QMainWindow>
 #include "aptinterface.h"
 
-
 class AptInterface;
 class PackageView;
 class Uninstall;
 class Dialog;
 class dimmer;
-
-
-const QString FAM_VERSION = "0.1";
+class RepoView;
 
 
 namespace Ui {
@@ -53,6 +50,9 @@ public:
 	void setOperation(operation op_) { iOperation=op_; }
 	void closeEvent(QCloseEvent *event);
 
+public slots:
+	void on_btnUpdate_clicked();
+
 protected:
     void changeEvent(QEvent *e);
 
@@ -60,17 +60,18 @@ private:
     Ui::MainWindow *ui;
 	AptInterface* iAptInterface;
 	PackageView* iWinPackageView;
+	RepoView* iWinRepoView;
 	Dialog* iDlgBusy;
 	operation iOperation;
 	dimmer* iDimmer;
 
 private slots:
+	void on_btnRepos_clicked();
 	void on_actionClean_triggered();
- void on_actionAbout_triggered();
+	void on_actionAbout_triggered();
 	void on_btnUpgrade_clicked();
 	void on_btnListInstallable_clicked();
 	void on_btnListInstalled_clicked();
-	void on_btnUpdate_clicked();
 };
 
 #endif // MAINWINDOW_H
